function processbatch(cfg, algcfg)
% Function to process a batch of DP/RL experiments

CFG.alg = [];                       % currently supported: llrfittedqi
CFG.mode = 'run';                   % 'run', 'sol', etc.
CFG.startat = 1;                    % offset for experiment index
CFG.repeat = 1;                     % repeat each experiment this many times
CFG.distribute = {};                % instead of being treated as expanded, these params should be
    % distributed one-by-one to the experiments; the same number of values as the number of
    % experiments must be provided
CFG.reusesamples = 0;               % whether to reuse samples:
    % 0 = do not reuse
    % 1 = all experiments in a given repetition use the same samples
    % 2 = experiments reuse samples from the batch with the config reusesamplescfg
    %   (data files are assumed available)
CFG.reusesamplescfg = [];
%
% experiment names and data saving configuration
CFG.idfields = [];                  % include ONLY these fields in the experiment id 
    % (by default, alg name, all showable&set parameters, and the repetition index are shown)
CFG.idprefix = '';
CFG.idinfix = '';
CFG.idpostfix = '';
CFG.dryrun = 0;                     % set to 1 to just test if all configs are properly ran
CFG.verb = 3;

cfg = parseconfig(cfg, CFG);
dispx('PROCESSBATCH will run with the following config:', cfg.verb, 1);
dispx(cfg, cfg.verb, 1, 'cfg', 'config');

algfun = eval(['@' cfg.alg '_exp']);

% get algorithm params and process algorithm configs
PARAMS = algfun('params');
[params, expconfigs] = processparams(algcfg, PARAMS, cfg.distribute);
nexp = length(expconfigs);

% if reusing samples from another batch, get params and algorithm configs
if cfg.reusesamples == 2,
    rs_cfg = parseconfig(cfg.reusesamplescfg, CFG);
    rs_PARAMS = feval([rs_cfg.alg '_exp'], 'params');
    % note for reuse samples it is not clear that distribute should have the same value as the main
    % config
    [rs_params, rs_expconfigs] = processparams(rs_cfg, rs_PARAMS, cfg.distribute);
end;

% process experiments
for rep = cfg.startat:(cfg.startat+cfg.repeat-1),
    dispx(sprintf('Processing set%d in mode [%s]', rep, cfg.mode), cfg.verb, 1);
    for iexp = 1:nexp,
        % pick up algorithm config
        expcfg = expconfigs{iexp};
        % generate ID; this is also the datafile name
        expid = generateexpid(cfg, params, expcfg, rep);
        expcfg.datafile = expid;
        if strcmp(cfg.mode, 'run')
            % no need to run algorithm if file already there
            if (exist([expid '.mat'], 'file')), 
                dispx(sprintf('Skipping set%d/exp%d, id=%s (datafile found).', rep, iexp, expid), cfg.verb, 4);
                continue; 
            end;
            % configure sample reuse
            if cfg.reusesamples == 1 && iexp > 1,   % reuse from previous experiment in the batch
                expcfg.loadsamples = generateexpid(cfg, params, expconfigs{iexp-1}, rep);
            elseif cfg.reusesamples == 2,           % reuse from similar experiment in another batch
                expcfg.loadsamples = generateexpid(rs_cfg, rs_params, rs_expconfigs{iexp}, rep);
                if ~exist([expcfg.loadsamples '.mat'], 'file'),
                    dispx(sprintf('WARNING! Should reuse samples from [%s], but file does not exist', ...
                        expcfg.loadsamples), cfg.verb, 0);
                end;
            end;
        end;
        dispx(sprintf('Processing set%d/exp%d, id=%s', rep, iexp, expid), cfg.verb, 3);
        % delegate call to algorithm handling function
        if ~cfg.dryrun, algfun(cfg.mode, expcfg); end;
    end;    % FOR experiments
end;        % FOR repetition


% end processbatch