%% Demonstration of DP/RL algorithms on the cleaning robot problem
% Initialization

% common config
ccfg.target = 'screen';
ccfg.problem = 'cleanrob_problem';
pparam.X = {0:5};
pparam.x_can = 5;
pparam.rew_energy = 1;
pparam.rew_can = 5;
ccfg.model_params = {pparam};
ccfg.gamma = .5;
ccfg.eps = .001;
ccfg.viscfg = struct;
ccfg.viscfg.figsize = [600 600];
ccfg.viscfg.pause = -1;

% to get only console output for LaTeX, use this:
% ccfg.verb = -Inf; ccfg.viscfg.consoleout = 1;


% retrieve (or, if not available, compute) optimal solution with policy iteration
cfg = ccfg;
cfg.datafile = 'cleanrob_optsol_pi';
if ~exist([cfg.datafile '.mat'], 'file'),
    cfg.run = 1;
    cfg.verb = -Inf;    % keep things silent
    piter(cfg);
end;
load(cfg.datafile, 'Q', 'h', 'Qh');
ccfg.viscfg.Qstar = Q;
ccfg.viscfg.hstar = h;
ccfg.viscfg.Qhstar = Qh;

% common parameters changes for online learning
onlcfg = ccfg;
onlcfg.eps = .001;
onlcfg.reset = [2 3];
onlcfg.explor = .3;
onlcfg.explordecay = 1;
onlcfg.alpha = .2;
onlcfg.viscfg.pause = {'cfg.trial <= 1', -1; 'true', 0};


%% Q-iteration
cfg = ccfg;
cfg.visualize = 1;
cfg.run = 1;
qiter(cfg);


%% Policy iteration with Q-functions
cfg = ccfg;
cfg.visualize = 2;
cfg.run = 1;
piter(cfg);

%% Planning, optimistic planning for deterministic systems
cfg = ccfg;
cfg.planalg = 'opd';
cfg.model_params{1}.rew_energy = 0.2;
cfg.model_params{1}.rew_can = 1;
cfg.planparams.n = 6;
cfg.planparams.U = [-1 1];
cfg.planparams.verb = 5;
cfg.planparams.reusetree = 0;
cfg.trials = 1;
cfg.maxsteps = 20;
cfg.reset = [2];
cfg.visualize = 2;

cfg.run = 1;
genmpc(cfg);


%% Monte Carlo "policy iteration" variant: returns are reset, exploring starts
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.explor = 1;
cfg.explordecay = 1;
cfg.peval_trials = 10;

cfg.visit = 'every';
cfg.explortype = 'es';
cfg.resetreturns = 1;               
% cfg.h0 = ones(1, 6);        % always go right
cfg.randseed = 100;        % uncomment for repeatable results
montecarlo(cfg);

%% Monte Carlo "policy iteration" variant: returns are reset, exploring every step
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.explor = 1;
cfg.explordecay = 1;
cfg.peval_trials = 10;

cfg.visit = 'every';
cfg.explortype = 'step';
cfg.resetreturns = 1;               
cfg.h0 = -ones(1, 6);        % always go left   
cfg.randseed = 100;        % uncomment for repeatable results
cfg.viscfg.pause = {'cfg.mcstep && cfg.trial <= 2', -1; 'true', 0};
montecarlo(cfg);

%% Monte Carlo "online" variant: online exploration, policy improved after every trial
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.peval_trials = 1;
cfg.visit = 'every';
cfg.explortype = 'step';
cfg.resetreturns = 0;
cfg.viscfg.pause = {'cfg.mctrial && cfg.trial <= 2', -1; 'true', 0};
% cfg.viscfg.pause = 0;
cfg.randseed = 10;        % uncomment for repeatable results
montecarlo(cfg);

%% SARSA
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.randseed = 10;        % uncomment for repeatable results
cfg.eps = -1;
sarsa(cfg);


%% Q-learning
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.randseed = 10;        % uncomment for repeatable results
cfg.eps = -1;
qlearn(cfg);

%% SARSA(lambda)
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.lambda = .5;
cfg.randseed = 1000;      % uncomment for repeatable results
cfg.eps = -1;
sarsa(cfg);



%% Q(lambda)-learning
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.lambda = .5;
cfg.randseed = 10;      % uncomment for repeatable results
cfg.eps = -1;
qlearn(cfg);


%% ER-Q-learning
cfg = onlcfg;
cfg.visualize = 3; 
cfg.run = 1;
cfg.maxtrials = 40;
cfg.randseed = 10;        % uncomment for repeatable results
cfg.ertype = 'fwd';
cfg.erN = 5;
cfg.viscfg.pause = {'any(cfg.trial == 3)', -1; 'true', 0};
cfg.eps = -1;
qlearn(cfg);



