function [cxp, u] = pidcontrol(cx, y, Ts, k, PID, ref)
% PID control. Controller state cx_k = [err_k-1 integrator_k]

% process 'init' mode
if ischar(cx) && strcmp(cx, 'init'),
    cxp = [0; 0];   % the initial state of the controller
    u = 2;          % the order of the controller
    return;
end;

% default gains and reference value
if nargin < 5,	PID = [1 0 0]; end;
if nargin < 6,  ref = 0; end;

% cx
% y
% Ts
% k
% PID
% ref

% a state vector might be fed instead of y, the index of the output is assumed 1
y = y(1);

cxp = 0 * cx;
err = ref - y;
prop = PID(1) * err;                % proportional term
int = cx(2) + PID(2) * err;         % integrator term
der = PID(3) * (err - cx(1)) / Ts;  % derivative term

% the next state: current error and integrator term
cxp(1) = err;
cxp(2) = int;

% the control value (output of the PID controller)
u = prop + int + der;
