function  [u, xseq, info, data] = masimplecontrol(x, model, cfg)
% Some simple state feedback control algorithms for multiagent systems
% Note that for type=flock this implementation is wasteful since each agent computes the full control law, but whatever
%
% Note XSEQ, INFO, DATA are placeholders to obey the planning algorithm template, here they are
% returned empty


CFG.type = 'rand';              % control type: 'rand'om, 'flock'ing basic
CFG.iagent = [];                % index of this agent
CFG.alpha = [];
CFG.G = [];                     % graph of interagent connections
% display config
CFG.verb = 0;                   % verbosity: the higher, the more detailed the messages displayed

cfg = checkparams(cfg, CFG);
x = reshape(x, model.p(1), model.nagents);
iagent = cfg.iagent;

if cfg.type(1) == 'r',
    u = rand(model.q(1), 1);
elseif cfg.type(1) == 'f',
    vel = x(model.dim+1:end, :);
    d = sum(cfg.G);         % degrees vector
    L = diag(d) - cfg.G;    % Laplacian amtrix
    if isempty(cfg.alpha),  alpha = 1/(max(d)+1);
    else                    alpha = cfg.alpha;
    end;
    U = (-alpha * L * vel')'; % control law
    u = U(:, iagent);
elseif cfg.type(1) == 'd'   % 'dubbins' model flocking
    theta = x(4, :);
    neighb = cfg.G(iagent, :); neighb(iagent) = 0; % make sure the agent is not its own neighbor
    if ~any(neighb), error('Agent [%d] is disconnected', iagent); end;
    omega = cfg.k * sum(theta(logical(neighb)) - theta(iagent));
    if abs(omega) > 1e20, error('Unstable inputs'); end;
    u = [0; omega];
end;

% no stats/data
xseq = []; info = struct; data = [];

end % main function


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local functions



