% Test discrete-time flocking with or without minimal robust subgraph condition satisfied
clc; clear all; close all;


%% Create model, initial config, compute robustness, and then several initial states

mcfg = struct;
mcfg.type = 'doubleint';
mcfg.nagents = 6;
mcfg.dim = 1;
mcfg.range = 5;
mcfg.flock_dist = 5;
mcfg.flock_maxdvel = 5;
% mcfg.flock_alpha = .25;
% mcfg.flock_beta = .75;
mcfg.Ts = 1;

m = malinear_problem('model', mcfg);

x0 = [0:0.4*mcfg.range:0.4*mcfg.range*(mcfg.nagents-1); zeros(1, mcfg.nagents)];
[Grob, rhorob, lambda2rob] = malinear_problem('minrobgraph', m, x0);

vrob = lambda2rob * rhorob / sqrt(2) / sqrt(m.nagents);
% case 1): condition satisfied
x0_1 = [x0(1, :); vrob * [-ones(1, m.nagents/2) ones(1, m.nagents/2)]];
% case 2): condition violated but with intermediate value
x0_2 = [x0(1, :); 1.5 * vrob * [-ones(1, m.nagents/2) ones(1, m.nagents/2)]];
% case 3): condition violated with large value
x0_3 = [x0(1, :); 2.5 * vrob * [-ones(1, m.nagents/2) ones(1, m.nagents/2)]];


%% Basic control algorithm
cfg = struct;
cfg.problem = 'malinear_problem';
cfg.model_params = {mcfg};
cfg.planalg = 'masimplecontrol';
cfg.planparams.type = 'flock';
cfg.visualize = 2;
cfg.viscfg.pause = .01;
cfg.datadir = pwd;
cfg.maxsteps = 100;
cfg.datafile = 'di_simplecontrol';

%% run in case 1
cfg.reset = x0_1(:); cfg.run = 1; magenmpc(cfg);
%% run in case 2
cfg.reset = x0_2(:); cfg.run = 1; magenmpc(cfg);
%% run in case 3
cfg.reset = x0_3(:); cfg.run = 1; magenmpc(cfg);


%% Optimistic planning
cfg = struct;
cfg.problem = 'malinear_problem';
cfg.model_params = {mcfg};
cfg.planalg = 'maopd_predictneighbors';
% cfg.planparams.U = repmat({symloggrid(11, .3)}, 1, mcfg.nagents);
cfg.planparams.U = repmat({[-.3 -.15 -.05 0 .05 .15 .3]}, 1, mcfg.nagents);
cfg.planparams.n = 500;
% cfg.planparams.gametype = 'max';
cfg.planparams.rob = 0;
cfg.planparams.maxpredlength = Inf;
cfg.planparams.verb = 2;
cfg.visualize = 2;
cfg.viscfg.pause = .01;
cfg.datadir = pwd;
cfg.maxsteps = 100;
cfg.verb = 3;
cfg.saveplandata = 1;
cfg.datafile = 'di_maopd_predneighb';

%% run in case 1
cfg.reset = x0_1(:); cfg.run = 1; magenmpc(cfg);

%% run in case 2
cfg.reset = x0_2(:); cfg.run = 1; magenmpc(cfg);
%% run in case 3
cfg.reset = x0_3(:); cfg.run = 1; magenmpc(cfg);

