function figh = mararm_plot(cfg)
% Plot multi-robot-arm system evolution

% default arguments
if nargin < 1, cfg = ''; end;
% support for default calling mode -- i.e. only history
if isstruct(cfg) && isfield(cfg, 't') && isfield(cfg, 'x'),
    hist = cfg; cfg = struct; cfg.traj = hist;
    cfg.datasource = 'caller';
end;

CFG.problem = 'mararm_problem';
CFG.datasource = '';
CFG.datafile   = '';    % data source can be given as a data file here as well
CFG.traj = [];          % plot history (given as standard history structure)
CFG.plotrew = 0;        % whether to plot agent rewards
CFG.subplots = [];
% plot configuration
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG

% process config
cfg = parseconfig(cfg, CFG);
if ~isempty(cfg.datafile), cfg.datasource = cfg.datafile; end;

% needed vars
vars = {'model', 'cfg'};
% load model, grids, etc. from the data source
cfg1 = cfg;
switch cfg.datasource,
    case 'caller',              % load needed vars from caller space
        for i=1:length(vars), cv.(vars{i}) = evalin('caller', vars{i}); end;
        structtovars(cv);
    otherwise                   % load from file
        load(cfg.datasource, vars{:});
end;
expcfg = cfg; cfg = cfg1; clear cfg1; % swap config

% color style
cs = struct;
cs.agentline = {{'b'}, {'r'}, {'g'}};
cs.return = {'Color', 'g', 'LineWidth', 1.5};
% grayscale style starting from color style
gs = cs;
gs.agentline = {{'k-'}, {'k:'}, {'Color', [.5 .5 .5]}};
% choose style
if cfg.grayscale, sty = gs; else sty = cs; end;
% labels
labels = struct;
labels.x = {'\theta_1', '\theta''_1', '\theta_2', '\theta''_2'}; 
labels.u = {'\tau_1, \tau_2'}; labels.r = 'r'; labels.time = 't [s]';


% %%%%%%%%%%%%%%%%%%%%%%
% PLOT DATA

figh = [];

if ~isempty(cfg.traj),
    figh(end+1) = figure;
    traj = cfg.traj;
    % pick up positions, velocities
    x = reshape(traj.x, model.p1, model.nagents, traj.Ns+1);
    u = reshape(traj.u, model.q1, model.nagents, traj.Ns+1);
    r = reshape(traj.r, 1, model.nagents, traj.Ns+1);
    if cfg.plotrew,
        subplots = {{3, 2, 1}, {3, 2, 3}, {3, 2, 2}, {3, 2, 4}, {3, 2, [5 6]}};
        values = [x; r];
        labels.values = {labels.x{:}, labels.r};
    else
        subplots = {{2, 2, 1}, {2, 2, 2}, {2, 2, 3}, {2, 2, 4}};
        values = x;
        labels.values = labels.x;
    end;
    
    if ~isempty(cfg.subplots), subplots = cfg.subplots; end;
    
    for v = 1:size(values, 1);
        subplot(subplots{v}{:});
        for i = 1:model.nagents,
            plot(traj.t, squeeze(values(v, i, :)), sty.agentline{i}{:}); 
            hold on;
        end;
        if (cfg.plotrew && v == size(values, 1)) || (~cfg.plotrew && v == 1),  % add legend
            leg = cell(model.nagents, 1); 
            for i = 1:model.nagents, leg{i} = ['robot ' num2str(i)]; end;
            lh = legend(leg);
            set(lh, 'Location', 'best');
        end;
        xlabel(labels.time);
        ylabel(labels.values{v});
    end;
    
end;

% save last figure if indicated
if ~isempty(figh), saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget); end;

end
% END RETURNING array of figure handles =================================================

