function figh = comparelspi(cfg)
% Compares two LSPI experiments.
% Currently only compares theta differences, and only for LSPIOPTIM versus LSPI
% Limited-use, "ad-hoc"

CFG.baseline = '';                  % baseline datafile
CFG.datafile = '';                  % compare to this datafile
CFG.comparison = 'thetadiff';       % 'thetadiff' is the only supported type currently
%
CFG = setfigprop(CFG, 'addfields');

cfg = parseconfig(cfg, CFG);


if any(strcmp(cfg.comparison, 'thetadiff')),
    % baseline: only final parameter vector is needed
    bvars = {'theta'};              
    % datafile: theta history required
    dvars = {'thetah', 'thetahk'};
end;

loadwithprefix(cfg.baseline, 'b_', bvars);
loadwithprefix(cfg.datafile, 'd_', dvars);


figh = [];

if any(strcmp(cfg.comparison, 'thetadiff')),
    thetadiff = nan(100, 1);        % a default size
    % iterate through history, taking care to check for early termination (i.e., empty theta)
    for i = 1:length(d_thetah),
        if isempty(d_thetah{i}), break;
        else    thetadiff(i) = norm(b_theta - d_thetah{i}, 2);
        end;
    end;
    if isempty(d_thetah{i}), n = i - 1; else n = i; end;
    thetadiff = thetadiff(1:n);

    figh(end+1) = figure;
    semilogy(d_thetahk(1:n), thetadiff, 'k');
    setfigprop(cfg);
end;