function bike_cerbfps(exper, mode, c)
% Cross-entropy policy search for the bicycle problem.
% Experiments are:
%   - run in Feb 2009, 
%   - with the updated version of CERBFPS 
%   - with new settings for the parameters (finer thresholds and larger #samples mainly)
%   - on the new DCSC desktop machines (Dell Optiplex 755)

% What experiments to process and in what mode (defaults)
if nargin < 1, exper = 'detp5'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% defaults
CFG = struct;
CFG.(mode) = 1;
CFG.startat = 1; CFG.repeat = 10;
CFG.problem = 'bike_problem'; 
CFG.gamma = 0.98;
CFG.alg = 'cerbfps';
CFG.N = []; % placeholder
% Note below changes from thesis&older experiments: 
% there c=5, rho=.05, alpha=1 (no smooth), eps=.005
CFG.ce_c = 10;          
CFG.ce_rho = 0.01;
CFG.ce_alpha = 0.7;
CFG.ce_eps = 0.001;
CFG.mc_maxsteps = 0.001;
% output config
CFG = setfigprop(CFG, 'addfields');
CFG.datadir = pwd;
CFG.verb = 2;
CFG.int = 'conf';
CFG.conftype = 'gauss';
CFG.show = {'rep'};
CFG.statspec = '%g';

% parse config
ccfg = parseconfig(c, CFG);

% === Deterministic experiment with small set of initial states
d1 = ccfg;
d1.miscinfo = 'detp5';
d1.model_params = {'det=1'};
d1.N = 3:8;
d1.X0 = 'p5';
d1.infix = 'det';
if checkrunexp(d1.miscinfo, exper), optimexp(d1); end;


% === Deterministic experiment with larger set of initial states
% Estimation on how long these experiments will take:
% runtime/exp * #values of N * #repetitions * incr #samples * incr horizon * incr #iter
% divided by 2 machines, each with 2 cores, and running 3 times as fast as the old ones
% total = (1.65e4 * 6 * 20) * 2 * (536/456) * 1.3; percore=total/2/2/3; days = percore/3600/24
d2 = ccfg;
d2.miscinfo = 'detp2s15';
d2.model_params = {'det=1'};
d2.N = 3:8;
d2.X0 = 'p2s15';
d2.infix = 'det';
if checkrunexp(d2.miscinfo, exper), optimexp(d2); end;


% === Stochastic experiment with small set of initial states
s1 = ccfg;
s1.miscinfo = 'stochp5';
s1.model_params = {'det=0'};
s1.N = 7;
s1.X0 = 'p5';
s1.mc_nsim = 10;
s1.infix = 'stoch';
if checkrunexp(s1.miscinfo, exper), optimexp(s1); end;


% === Stochastic experiment with larger set of initial states
s2 = ccfg;
s2.miscinfo = 'stochp2s15';
s2.model_params = {'det=0'};
s2.N = 7;
s2.X0 = 'p2s15';
s2.mc_nsim = 10;
s2.infix = 'stoch';
if checkrunexp(s2.miscinfo, exper), optimexp(s2); end;



