function figh = dcconv_lqr_lin(cfg)
% Convergence study synch vs. asynch for the DC motor
% Uses the most refined state & action gridding from the consistency experiments
% Written for the Fuzzy Q-iter jnl paper draft

CFG = struct;
CFG = setfigprop(CFG, 'addfields');
CFG.plottarget = 'latex';
CFG.savedir = 'D:\Work\tex\papers\fzqi\img\';
CFG.datadir = 'd:\work\m\approxrl\systems\dcmotor\dc_conv_fzvsrbf_lqr_lin\';
CFG.savefig = 'dc_conv';
CFG.figsize = [450 350];


CFG.N = 41;
CFG.M = 15;
% create the following files if needed
CFG.pfile = 'fzdc_lqr_lin_par'; 
CFG.sfile = 'fzdc_lqr_lin_ser'; 
CFG.qieps = 1e-5;
CFG.plottarget = 'latex';

cfg = parseconfig(cfg, CFG);

crtdir = pwd;
cd(cfg.datadir);

% common config
ccfg = struct;
ccfg.problem = 'dc_problem';
ccfg.xgrids = {-pi:2*pi/(cfg.N-1):pi, -16*pi:32*pi/(cfg.N-1):16*pi};
ccfg.ugrids = {-10:20/(cfg.M-1):10};
ccfg.gamma = 0.95;
ccfg.model_params = {'lqr', 'lin'};
ccfg.savetheta = 1;
ccfg.datadir = cfg.datadir;
% run with better precision than actually required to get an estimate of the optimal theta
ccfg.eps = 1e-3 * cfg.qieps;   
ccfg.plottarget = cfg.plottarget;
ccfg.savedir = cfg.savedir;

% parallel config
if ~exist([cfg.pfile '.mat'], 'file')
    pcfg = ccfg;
    pccfg.serial = 0;
    pccfg.run = 1;
    pccfg.datafile = cfg.pfile;

    fuzzyqi(pcfg);
end;

% serial config
if ~exist([cfg.sfile '.mat'], 'file')
    scfg = ccfg;
    sccfg.serial = 1;
    sccfg.run = 1;
    sccfg.loadapprox = cfg.pfile;
    sccfg.datafile = cfg.sfile;
    sccfg.iterdisp = 1;
    sccfg.itersave = 2;

    fuzzyqi(scfg);
end;

load([cfg.pfile '.mat'], 'theta', 'qistats');
pstats = qistats;
pth = theta;

load([cfg.sfile '.mat'], 'theta', 'qistats');
sstats = qistats;
sth = theta;

% use the result of parallel as the approximate convergence point
thstar = pth;

for i = 1:length(pstats.delta)+1,
    pd(i) = max(max(abs(pstats.theta{i} - thstar)));
end;
pk = find(pd < cfg.qieps, 1, 'first');
pk
for i = 1:length(sstats.delta)+1,
    sd(i) = max(max(abs(sstats.theta{i} - thstar)));
end;
sk = find(sd < cfg.qieps, 1, 'first');
sk

figh = figurex();
% commonprop = {'Interpreter', 'Latex', 'FontSize', 13};
commonprop = {};
semilogy(0:pk-1, pd(1:pk), 'k-'); hold on;
semilogy(0:sk-1, sd(1:sk), 'k--', 'LineWidth', 1.5); hold on;
% Latex labels: no longer used
% xlabel('Iteration $\ell$', commonprop{:}); 
% ylabel('$\Vert \theta_\ell - \hat{\theta}^* \Vert_\infty$', commonprop{:});
xlabel('Iteration number'); 
ylabel('Distance to near-optimal parameters');
legend('Synchronous', 'Asynchronous'); 
set(gca, 'ylim', [1e-5-1e-7 1e3]);
set(gca, 'ytick', 10.^[-5:2:3]);
setfigprop(cfg);

saveplot(figh, [cfg.savedir cfg.savefig], cfg.plottarget);

cd(crtdir);