% This were the original experiments, superseded now by experiments using
%   gamma = 0.97
%   "standard" Q and R
%   ortho approximators instead of poly
%   M comparison with adaptive Ns, instead of Ns only

% Note that plotting scripts (for reports etc) may rely on this file & the associated data to
% be in the parent directory

% What experiments to process and in what mode
exp = {'*nil','*pd'};
% exp = 'all';
mode = 'plot';       % 'run', 'sol', 'stats', 'cleanup'

% main cfg
cfg = struct;
cfg.(mode) = 2;
cfg.gamma = .95;
cfg.problem = 'dc_problem';
cfg.gamma = .95;
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model
cfg.samesamples = 1;
cfg.datadir = pwd;
cfg.Ns = [5000 10000 15000];      % for this # samples, available 5 exp for NIL and OPT
% cfg.Ns = 10000;     
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.maxiter  = 100;
cfg.repeat = 5;
cfg.infix = 'h0nil';
% perf eval
cfg.perf = 1;                       % turn off for now, we care about iter only
cfg.X0 = 'coarse';                  % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;      
% replay
cfg.x0 = [pi;0]; cfg.tend = 1.5;
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'};
    if any(strcmp(exp, '*opt')),
        cfg.savefig = 'dch0opt';
    elseif any(strcmp(exp, '*pd')),
        cfg.savefig = 'dch0pd';
    else
        cfg.savefig = 'dch0';
    end;
    cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
%     cfg.plottarget = 'latex'; 
    cfg.xlim = [min(cfg.Ns) max(cfg.Ns)];
    cfg.xtick = cfg.Ns;
    cfg.ylim_matrix = [-267 -261; 0 30];
    cfg.figsize = [450 350];
end;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

% Config for using OPTIMAL policy (discounted LQR controller)
model = dc_problem('model', cfg.model_params{:}); 
K = gammadare(struct('model', model, 'gamma', cfg.gamma));
hcfg = cfg;
hcfg.lspi_params.h0 = @lsf;
hcfg.lspi_params.h0args = {K, model.maxu};
hcfg.infix = 'h0opt';
% Config for using PD policy (discounted LQR controller)
hcfg_pd = cfg;
hcfg_pd.lspi_params.h0 = @pdstatic;
hcfg_pd.lspi_params.h0args = {[21.1765 0.6514], model.maxu, 0};
hcfg_pd.infix = 'h0pd';


expout = [];        % reset figure additions
% -------- TRIANG experiments
% TRIANGULAR discrete / optimal policy
thexp = hcfg;
thexp.type     = 'triang'; thexp.miscinfo = 'th_opt';
thexp.N        = 9; thexp.M = 5;
thexp.method   = 'randdisc';
thexp.addtocrt = 0; thexp.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(thexp.miscinfo, exp), expout = lspiexp(thexp); end;
% TRIANGULAR discrete / PD policy
thexp_pd = hcfg_pd;
thexp_pd.type     = 'triang'; thexp_pd.miscinfo = 'th_pd';
thexp_pd.N        = 9; thexp_pd.M = 5;
thexp_pd.method   = 'randdisc';
thexp_pd.usesamples = thexp;
thexp_pd.addtocrt = 0; thexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(thexp_pd.miscinfo, exp), expout = lspiexp(thexp_pd); end;
% TRIANGULAR discrete
texp = cfg;
texp.type     = 'triang'; texp.miscinfo = 't_nil';
texp.N        = 9; texp.M = 5; 
texp.method   = 'randdisc';
texp.addtocrt = expout; texp.expprefix = 'no h_0,';     % only effective in stats+plot mode
texp.usesamples = thexp;
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;

expout = [];        % reset figure additions
% TRIANGULAR quad / optimal policy
tphexp = hcfg;
tphexp.type     = 'triangpoly'; tphexp.miscinfo = 'tph_opt';
tphexp.N        = 9; tphexp.M = 2;
tphexp.method   = 'rand';
tphexp.addtocrt = 0; tphexp.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(tphexp.miscinfo, exp), expout = lspiexp(tphexp); end;
% TRIANGULAR quad / PD policy
tphexp_pd = hcfg_pd;
tphexp_pd.type     = 'triangpoly'; tphexp_pd.miscinfo = 'tph_pd';
tphexp_pd.N        = 9; tphexp_pd.M = 2;
tphexp_pd.method   = 'rand';
tphexp_pd.usesamples = tphexp;
tphexp_pd.addtocrt = 0; tphexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(tphexp_pd.miscinfo, exp), expout = lspiexp(tphexp_pd); end;
% TRIANGULAR quad
tpexp = cfg;
tpexp.type     = 'triangpoly'; tpexp.miscinfo = 'tp_nil';
tpexp.N        = 9; tpexp.M = 2;
tpexp.method   = 'rand';
tpexp.usesamples = tphexp;
tpexp.addtocrt = expout; tpexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(tpexp.miscinfo, exp), expout = lspiexp(tpexp); end;

expout = [];        % reset figure additions
% -------- RBF experiments, regular inversion
% RBF discrete H
rhexp = hcfg;
rhexp.type     = 'rbfdisc'; rhexp.miscinfo = 'rh_opt';
rhexp.N        = 9; rhexp.M = 5;
rhexp.method   = 'randdisc';
rhexp.usesamples = thexp;
rhexp.addtocrt = 0; rhexp.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(rhexp.miscinfo, exp), expout = lspiexp(rhexp); end;
% RBF discrete / PD policy
rhexp_pd = hcfg_pd;
rhexp_pd.type     = 'rbfdisc'; rhexp_pd.miscinfo = 'rh_pd';
rhexp_pd.N        = 9; rhexp_pd.M = 5;
rhexp_pd.method   = 'randdisc';
rhexp_pd.usesamples = thexp;
rhexp_pd.addtocrt = 0; rhexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(rhexp_pd.miscinfo, exp), expout = lspiexp(rhexp_pd); end;
% RBF discrete
rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r_nil';
rexp.N        = 9; rexp.M = 5; 
rexp.method   = 'randdisc';
rexp.usesamples = thexp;
rexp.addtocrt = expout; rexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

expout = [];        % reset figure additions
% RBF poly H
rphexp = hcfg;
rphexp.type     = 'rbfpoly'; rphexp.miscinfo = 'rph_opt';
rphexp.N        = 9; rphexp.M = 2;
rphexp.method   = 'rand';
rphexp.usesamples = tphexp;
rphexp.addtocrt = 0; rphexp.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(rphexp.miscinfo, exp), expout = lspiexp(rphexp); end;
% RBF poly / PD policy
rphexp_pd = hcfg_pd;
rphexp_pd.type     = 'rbfpoly'; rphexp_pd.miscinfo = 'rph_pd';
rphexp_pd.N        = 9; rphexp_pd.M = 2;
rphexp_pd.method   = 'rand';
rphexp_pd.usesamples = tphexp;
rphexp_pd.addtocrt = 0; rphexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(rphexp_pd.miscinfo, exp), expout = lspiexp(rphexp_pd); end;
% RBF poly
rpexp = cfg;
rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rp_nil';
rpexp.N        = 9; rpexp.M = 2;
rpexp.method   = 'rand';
rpexp.usesamples = tphexp;
rpexp.addtocrt = expout; rpexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;



