function lspiodc_trialtime(exp, mode, c)
% Real inverted pendulum. Using trajectories for samples

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'ronl'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
cfg.(mode) = 2;
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.gamma = .97;
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model% misc config (output etc)
cfg.datadir = pwd;
cfg.eps = 0;                            % zero convergence threshold (force until real convergence)
% approximator main config
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.datadir = pwd;
% perf eval
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.plottarget = 'screen'; cfg.savefig = 'tt';
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J'}; 
    cfg.figsize = [450 350];
end;
% if strcmp(mode, 'onlineperf'), 
%     cfg.ylim = [-2400 -1400];
% end;
% if any(strcmp(mode, {'stats', 'plot', 'onlineperf'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.alg = 'lspionline';
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.lspi_params.explor = 1;                % start with this exploration rate
ocfg.explordecay = 0.1^(1/ocfg.lspi_params.maxtime);
ocfg.dupd = 100 * 0.005;                    % 100 steps, different from IPSETUP where we chose 10 steps!
ocfg.lspi_params.dstoretheta = 1;           % store theta once every second
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'trialtime', 'rep'};
ocfg.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
% perf eval
ocfg.forceeval = 1;                         % since no experiment actually converges

expout = [];   % dummy

% ONLINE, RBF discrete experiment
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl';
rexp.N        = 9; rexp.M = 3;
rexp.trialtime = [0.3 0.75 1.5 3 6 12]; 
rexp.addtocrt = []; rexp.expprefix = 'Online discrete'; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% ONLINE, RBF ORTHO experiment
roexp = ocfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'roonl';
roexp.N        = 9; roexp.M = 2;
roexp.trialtime = [0.3 0.75 1.5 3 6 12]; 
roexp.addtocrt = []; roexp.expprefix = 'Online ortho'; % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;


end



