function dd_lspi_resstudy(exp, mode, cfg)
% Discrete double integrator. Resolution study

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'basic'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, cfg = struct; end;

% NOTE the LSPI conv threshold is not directly comparable to the PS threshold: one is in
% parameter space, the other in actual returns

% default (main) CFG
CFG = struct;
CFG.(mode) = 1;
CFG.startat = 1; CFG.repeat = 20;
CFG.problem = 'ddi_problem';
CFG.gamma = 0.95;
CFG.maxiter  = 100;
CFG.eps = .001; CFG.epsper = .001; CFG.maxper = 15;
% approx and samples
CFG.type = 'rbfdisc';
CFG.N = 4:18;
CFG.xspace = 'eq'; 
CFG.uspace = 'pre';                 % pre-defined action space (don't create dynamically, only use this)
CFG.U = {[-.1 .1]};
CFG.method   = 'randdisc';
CFG.adaptiveNs = 1; CFG.Ns = 1000;  % baseline Ns
% perf eval
CFG.perf = 1; CFG.X0 = 'p01s02'; CFG.mc_eps = 0.001;
CFG.datadir = pwd;
CFG.hide = 'M';
% don't clutter the main cache with these scores, keep a local cache
CFG.pcache = 'd:\work\m\approxrl\systems\discretedi\dd_lspi_resstudy\dd_lspi_resstudy_localcache.mat';        
% figures
CFG.plotminmax = 0; CFG.plotconf = 1; CFG.minmaxtype = 'area'; CFG.transparency = 0;
CFG.conftype = 'gauss';
CFG.expprefix = '';
CFG.fixedfigname = 0;
if strcmp(mode, 'plot'),
    CFG.stats = 1; CFG.plot = {'J'};
end;
if any(strcmp(mode, {'stats', 'plot'})), CFG.verb = 0; else CFG.verb = 5; end;

% install any non-default config; preserve the default
cfg = parseconfig(cfg, CFG);


% ==== the basic experiment
bcfg = cfg;
bcfg.miscinfo = 'basic';
if checkrunexp(bcfg.miscinfo, exp), expout = lspiexp(bcfg); end;


% ==== a second experiment, where the # of samples is threefold 
% in order to test for its influence on the solution performance
% (Currently no figure is saved anywhere; the performance profile is similar to the basic
% experiment. Conclusion: number of samples is large enough in the basic experiment.)
largecfg = cfg;
largecfg.miscinfo = 'largeNs';
largecfg.Ns = 3000;     % triple the # of samples for the basic experiment
largecfg.repeat = 2;    % a few test-runs (at least for now)
largecfg.infix = 'largeNs';
if checkrunexp(largecfg.miscinfo, exp), expout = lspiexp(largecfg); end;

% ==== an experiment which fixes the termination problems (May 2010)
ncfg = cfg;
ncfg.miscinfo = 'nosat';
ncfg.model_params = {'QUAD2', 1, 'nosat'};
ncfg.infix = 'nosat';
ncfg.lspi_params.term = 'zero';
if checkrunexp(ncfg.miscinfo, exp), expout = lspiexp(ncfg); end;

end