function hiv_cerbfps(mode, c)
% Cross-entropy policy search for the discrete double integrator problem.
% Experiments are:
%   - run in Feb 2009, 
%   - with the updated version of CERBFPS 
%   - on the new DCSC desktop machines (Dell Optiplex 755)

% What experiments to process and in what mode (defaults)
if nargin < 1, mode = 'run'; end;
if nargin < 2, c = struct; end;

% defaults
CFG = struct;
CFG.(mode) = 1;
CFG.startat = 1; CFG.repeat = 1;
CFG.problem = 'hiv_problem';
CFG.gamma = 0.99;
CFG.alg = 'cerbfps';
CFG.actsel = 'max';
CFG.N = 8;
CFG.X0 = 'unh';
CFG.ce_c = 10;          
CFG.ce_rho = 0.01;      % this was .02 for the original experiments
CFG.ce_alpha = 0.7;
CFG.ce_eps = 0.0001;
CFG.ce_d = 5;
CFG.mc_maxsteps = 160;
CFG.maxiter = 100;
% output config
CFG.datadir = pwd;
CFG.show = {'rep'};
% CFG.postfix = '';
CFG.verb = 3;
CFG.int = 'conf';
% CFG = setfigprop(CFG, 'addfields');

% process config
ccfg = parseconfig(c, CFG);

% experiment (provide possibility to change defaults above)
cfg = ccfg; 
optimexp(cfg);


