function ip_Tsstudy

% study the influence of the sample time on the solution (sampling artifacts:)
% cfg.Ts = [0.1 0.075 0.05 0.04 0.03 0.02 0.01 0.005];
% cfg.Ts = [0.1 0.075 0.05 0.02 0.005]; % coarser version
% a good sample time
cfg.Ts = 0.02;
% cfg.Ts = 0.02;
% study the influence of the discount factor: coarse
% cfg.gamma = [0.95 0.96 0.97 0.98 0.99];
% fine around the "breaking" point for Ts=0.02 where convergence of LSPI is lost
% cfg.gamma = [0.97 0.975 0.976 0.977 0.978 0.98];
cfg.gamma = 0.9;       % single gamma, or reference gamma for the first sample time
cfg.adaptivegamma = 0;
cfg.loadapproxgamma =[];

% what to output
cfg.fzh = 1;
cfg.fzv = 0;
cfg.fzreplay = 1;
cfg.lspisol = 0;
cfg.lspireplay = 0;

% repeat gamma or Ts one according to the other to get the experiment settings
if isscalar(cfg.gamma) && ~isscalar(cfg.Ts),
    if cfg.adaptivegamma,
        % change gamma values such that the time horizon remains the same as for the first gamma/Ts
        % combination
        alpha = cfg.gamma ^ (1/cfg.Ts(1));
        cfg.gamma = alpha .^ cfg.Ts;
    else
        cfg.gamma = cfg.gamma + 0*cfg.Ts;
    end;
end;
if isscalar(cfg.Ts) && ~isscalar(cfg.gamma),
    cfg.Ts = cfg.Ts + 0*cfg.gamma;
end;

% echo config
cfg

for i = 1:length(cfg.Ts),
    % ========================
    % run fuzzy Q-iteration
    df = gendfname('fuzzyqi', cfg.Ts(i), cfg.gamma(i), 'u2');
    if (cfg.fzh || cfg.fzreplay) && ~exist([df '.mat'], 'file'),     % need to run experiment
        rcfg = struct;
        rcfg.run = 1;
        rcfg.gamma = cfg.gamma(i);
        rcfg.eps = 1e-2; 
        rcfg.maxiter = 3000;
        rcfg.problem = 'invpend_problem';
        rcfg.datafile = df;
        rcfg.fuzzy_params = {'fine'};
        rcfg.model_params = {'lqr', 'type=rot maxu=2', struct('Ts', cfg.Ts(i))};
%         rcfg.model_params = {'lqr', 'rot', struct('Ts', cfg.Ts(i))};
        if ~isempty(cfg.loadapproxgamma),
            rcfg.loadapprox = gendfname('fuzzyqi', cfg.Ts(i), cfg.loadapproxgamma);
        end; 
        fuzzyqi(rcfg);
    end;

    % plot fuzzy policy
    if cfg.fzh,
        pcfg = struct; pcfg.fuzzyh = 1; pcfg.datafile = df;
        dc_plot(pcfg);
    end;
    % plot VF
    if cfg.fzv,
        pcfg = struct; pcfg.fuzzyv = 1; pcfg.datafile = df;
        dc_plot(pcfg);
    end;

    % replay from stable eq.
    if cfg.fzreplay,
        recfg = struct;
        recfg.replay = 1;
        recfg.problem = 'invpend_problem';
        recfg.tend = 5;
        recfg.datafile = df;
        recfg.x0 = [pi, 0];
        recfg.interph = 0;
        fuzzyqi(recfg);
    end;

    % ========================
    % Run LSPI
    acfg.enhanced = 1;
%     acfg.type = 'rbfdisc'; 
%     acfg.type = 'rbfquad'; 
    acfg.type = 'triang'; 
    acfg.N = 11; acfg.M = 3; acfg.xspace = 'eq'; acfg.uspace = 'log';
    scfg.N = 10000; scfg.method = 'randdisc';
    % run
    df = gendfname('lspi', cfg.Ts(i), cfg.gamma(i), 'ri', acfg, scfg);
    if (cfg.lspisol || cfg.lspireplay) && ~exist([df '.mat'], 'file'),     % need to run experiment
        rcfg = struct;
        rcfg.run = 1;
        rcfg.gamma = cfg.gamma(i);
        rcfg.eps = 0.1; 
        rcfg.maxiter = 100;
        rcfg.problem = 'invpend_problem';
        rcfg.method = 'recinv';
        rcfg.datafile = df;
        rcfg.model_params = {'lqr', 'rot', struct('Ts', cfg.Ts(i))};
        rcfg.approx = acfg;
        rcfg.samples = scfg;
        lspi(rcfg);
    end;

    % plot solution incl. policy
    if cfg.lspisol,
        scfg = struct; scfg.sol = 1; scfg.datafile = df;
        lspi(scfg);
    end;

    % replay from stable eq.
    if cfg.lspireplay,
        recfg = struct;
        recfg.replay = 1;
        recfg.problem = 'invpend_problem';
        recfg.tend = 5;
        recfg.datafile = df;
        recfg.x0 = [pi,0];
        lspi(recfg);
    end;

end;    % FOR experiments

end         % FUNCTION ip_Tsstudy()


% -------------------
% Local functions

% Generate data file name
function df = gendfname(alg, Ts, gamma, infix, acfg, scfg)
% acfg, scfg only need to be specified for lspi
    if nargin < 4, infix = ''; end;
    if nargin < 5, acfg = []; scfg = []; end;
    switch alg,
        case 'fuzzyqi',
            df = 'fzip_fine';
            if ~isempty(infix), df = [df '_' infix]; end;
        case 'lspi',
            df = 'lspiip';
            if ~isempty(infix), df = [df '_' infix]; end;
            df = [df '_' acfg.type 'N' num2str(acfg.N) acfg.xspace];
            switch acfg.type,
                case {'triang', 'rbfdisc'},
                    df = [df 'M' num2str(acfg.M) acfg.uspace];
                case {'triangquad', 'rbfquad'}
                    % do nothing
            end;
            df = [df '_' scfg.method 'Ns' num2str(scfg.N)];
    end;
    % gamma and Ts are common
    str = num2str(gamma); str = str(str ~= '.');
    df = [df '_g' str];
    str = num2str(Ts); str = str(str ~= '.');
    df = [df '_Ts' str];
end
