% Overactuated inverted pendulum (no swing is necessary to stabilize)
function lspiip_overact(exp, mode, c)
% Real inverted pendulum. Comparison of approximators

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% main cfg
cfg = struct;
cfg.(mode) = 2;
cfg.problem = 'invpend_problem';
% this should not give convergence problems, while still giving a time horizon for a good solution
cfg.gamma = 0.91;
cfg.maxiter  = 100;
cfg.repeat = 5; cfg.startat = 1;
cfg.xspace = 'eq'; cfg.uspace = 'log';
% cfg.Ns = [5000 10000 15000];  % DISC and POLY experiments available for these fixed Ns values
cfg.Ns = 10000;
cfg.samesamples = 1;
cfg.model_params = {'lqr', 'type=rot maxu=2', 'Ts=0.02'};
cfg.infix = 'u2';
cfg.show = {'gamma', 'rep'};
cfg.datadir = pwd;
% perf eval
cfg.perf = 1; cfg.forcerecompute = 0;
cfg.X0 = 'coarse';
cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 5;    % for replaying
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'}; 
    cfg.savefig = 'ipapprox';
%     cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
    cfg.savedir = 'D:\Work\tex\reports\2008-03-03_3wr\img\';
    cfg.plottarget = 'screen'; 
    cfg.xlim = [3 6];
    cfg.xtick = 3:6;
    cfg.ylim_matrix = [-222 -211; 0 100];
    cfg.figsize = [450 350];
end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

% adaptive-Ns base configuration
cfg_Ns = cfg;
cfg_Ns.samesamples = 0; cfg_Ns.adaptiveNs = 1;
cfg_Ns.Ns = 10000;                 % baseline for adaptive Ns

expout = [];    % dummy

% ==============================
% ===== Non-adaptive Ns
% Triang approx
tpexp = cfg;
tpexp.type     = 'triangpoly'; tpexp.miscinfo = 'tp';
tpexp.N        = 11; tpexp.M = [2 3 4];
tpexp.method   = 'rand';
tpexp.addtocrt = 0; tpexp.expprefix = 'poly,';     % only effective in stats+plot mode
if checkrunexp(tpexp.miscinfo, exp), expout = lspiexp(tpexp); end;
    
texp = cfg;
texp.type     = 'triang'; texp.miscinfo = 't';
texp.N        = 11; texp.M = 3;
texp.method   = 'randdisc';
texp.addtocrt = expout; texp.expprefix = 'discrete (M=3),'; % only effective in stats+plot mode
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;

% RBF approx
rpexp = cfg;
rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rp';
rpexp.N        = 11; rpexp.M = [2 3 4];
rpexp.method   = 'rand';
rpexp.usesamples = tpexp;
rpexp.addtocrt = 0; rpexp.expprefix = 'poly,'; % only effective in stats+plot mode
if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;

rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r';
rexp.N        = 11; rexp.M = 3;
rexp.method   = 'randdisc';
rexp.usesamples = texp;
rexp.addtocrt = expout; rexp.expprefix = 'discrete (M=3),'; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% ==============================
% ==== Adaptive Ns -- reuses some of the data files above, so those should be run first
% Triang approx
tpexp_Ns = cfg_Ns;      % poly
tpexp_Ns.type     = 'triangpoly'; tpexp_Ns.miscinfo = 'tpNs';
tpexp_Ns.N        = 11; tpexp_Ns.M = [2 3 4 5];
tpexp_Ns.method   = 'rand';
tpexp_Ns.addtocrt = 0; tpexp_Ns.expprefix = 'poly,';     % only effective in stats+plot mode
if checkrunexp(tpexp_Ns.miscinfo, exp), expout = lspiexp(tpexp_Ns); end;
    
texp_Ns = cfg_Ns;       % disc
texp_Ns.type     = 'triang'; texp_Ns.miscinfo = 'tNs';
texp_Ns.N        = 11; texp_Ns.M = [3 5];
texp_Ns.method   = 'randdisc';
texp_Ns.addtocrt = expout; texp_Ns.expprefix = 'discrete,'; % only effective in stats+plot mode
if checkrunexp(texp_Ns.miscinfo, exp), expout = lspiexp(texp_Ns); end;

toexp_Ns = cfg_Ns;          % ortho
toexp_Ns.type     = 'triangortho'; toexp_Ns.miscinfo = 'toNs';
toexp_Ns.N        = 11; toexp_Ns.M = [2 3 4 5]; 
toexp_Ns.orthotype = 'cheby';
toexp_Ns.method   = 'rand'; 
toexp_Ns.usesamples = tpexp_Ns;
toexp_Ns.addtocrt = expout; toexp_Ns.expprefix = 'ortho,';     % only effective in stats+plot mode
if checkrunexp(toexp_Ns.miscinfo, exp), expout = lspiexp(toexp_Ns); end;

% RBF approx
rpexp_Ns = cfg_Ns;      % poly
rpexp_Ns.type     = 'rbfpoly'; rpexp_Ns.miscinfo = 'rpNs';
rpexp_Ns.N        = 11; rpexp_Ns.M = [2 3 4 5];
rpexp_Ns.method   = 'rand';
rpexp_Ns.usesamples = tpexp_Ns;
rpexp_Ns.addtocrt = 0; rpexp_Ns.expprefix = 'poly,'; % only effective in stats+plot mode
if checkrunexp(rpexp_Ns.miscinfo, exp), expout = lspiexp(rpexp_Ns); end;

rexp_Ns = cfg_Ns;       % disc
rexp_Ns.type     = 'rbfdisc'; rexp_Ns.miscinfo = 'rNs';
rexp_Ns.N        = 11; rexp_Ns.M = [3 5];
rexp_Ns.method   = 'randdisc';
rexp_Ns.usesamples = texp_Ns;
rexp_Ns.addtocrt = expout; rexp_Ns.expprefix = 'discrete,'; % only effective in stats+plot mode
if checkrunexp(rexp_Ns.miscinfo, exp), expout = lspiexp(rexp_Ns); end;

roexp_Ns = cfg_Ns;            % ortho
roexp_Ns.type     = 'rbfortho'; roexp_Ns.miscinfo = 'roNs';
roexp_Ns.N        = 11; roexp_Ns.M = [2 3 4 5]; roexp_Ns.orthotype = 'cheby';
roexp_Ns.method   = 'rand';
roexp_Ns.usesamples = tpexp_Ns;
roexp_Ns.addtocrt = expout; roexp_Ns.expprefix = 'ortho,'; % only effective in stats+plot mode
if checkrunexp(roexp_Ns.miscinfo, exp), expout = lspiexp(roexp_Ns); end;
