function lspis_cmpapprox(exp, mode, c)
% Real inverted pendulum. Comparison of approximators

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
cfg.(mode) = 1.5;
cfg.problem = 'ipsetup_problem';
% this should not give convergence problems, while still giving a time horizon for a good solution
cfg.gamma = 0.98;
cfg.maxiter  = 150;
cfg.startat = 1; cfg.repeat = 5;
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.adaptiveNs = 1;
cfg.Ns = 10000;         % baseline Ns
cfg.samesamples = 0;    % instead, load among experiments
cfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
cfg.datadir = pwd;
% perf eval
cfg.plotminmax = 1; cfg.plotconf = 0;
cfg.perf = 1; cfg.X0 = 'coarse'; cfg.mc_eps = 0.1;
% replay
cfg.x0 = [pi;0]; cfg.tend = 5;    % for replaying
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.minmaxtype = 'line';
    cfg.plot = {'J', 'iter'}; 
    cfg.savefig = 'ipapprox';
    cfg.savedir = pwd;
    cfg.plottarget = 'screen'; 
    cfg.xlim = [3 6];
    cfg.xtick = 3:6;
    cfg.ylim_matrix = [];
%     cfg.ylim_matrix = [-225 -210; 0 100];
    cfg.figsize = [450 350];
end;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);
expout = 0;   % dummy

% ==========================================
% Used in book

% RBF discrete, 2nd experiment. Run on new DCSC desktop. Equid spacing of
% the action.
% 0 experiments available
rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r2';
rexp.N        = 11; rexp.M = [3 5]; rexp.uspace = 'eq';
rexp.method   = 'randdisc';
rexp.infix    = 'disc2';
% rexp.addtocrt = expout; rexp.expprefix = 'discrete-action, '; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% RBF ortho, 2nd experiment. The only difference is this is run on new
% DCSC desktops + this one doesn't contain a set for M = 5
% 0 experiments available
roexp = cfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'ro2';
roexp.N        = 11; roexp.M = [2 3 4]; roexp.orthotype = 'cheby';
roexp.method   = 'rand';
roexp.infix    = 'ortho2';
% roexp.addtocrt = expout; roexp.expprefix = 'continuous-action, '; % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;

% ==========================================
% Below used in thesis chapter

% RBF discrete
% 20 experiments available
rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r';
rexp.N        = 11; rexp.M = [3 5];
rexp.method   = 'randdisc';
rexp.addtocrt = expout; rexp.expprefix = 'discrete,'; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% Orthogonal polynomial RBF
% 20 experiments available
roexp = cfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'ro';
roexp.N        = 11; roexp.M = [2 3 4 5]; roexp.orthotype = 'cheby';
roexp.method   = 'rand';
roexp.addtocrt = expout; roexp.expprefix = 'orthogonal polynomial,'; % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;

% ==========================================
% Below not currently used (except reports etc.)

% RBF poly
% 5 experiments available
rpexp = cfg;
rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rp';
rpexp.N        = 11; rpexp.M = [2 3 4 5];
rpexp.method   = 'rand';
rpexp.usesamples = roexp;
rpexp.addtocrt = 0; rpexp.expprefix = 'poly,'; % only effective in stats+plot mode
if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;

% Triang discrete
% 5 experiments available
texp = cfg;
texp.type     = 'triang'; texp.miscinfo = 't';
texp.N        = 11; texp.M = [3 5];
texp.usesamples = rexp;
texp.method   = 'randdisc';
texp.addtocrt = 0; texp.expprefix = 'discrete,'; % only effective in stats+plot mode
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;

% Triang poly
% 5 experiments available
tpexp = cfg;
tpexp.type     = 'triangpoly'; tpexp.miscinfo = 'tp';
tpexp.N        = 11; tpexp.M = [2 3 4 5];
tpexp.method   = 'rand';
tpexp.usesamples = roexp;
tpexp.addtocrt = expout; tpexp.expprefix = 'poly,';     % only effective in stats+plot mode
if checkrunexp(tpexp.miscinfo, exp), expout = lspiexp(tpexp); end;

% Orthogonal polynomial triang
% 5 experiments available
toexp = cfg;
toexp.type     = 'triangortho'; toexp.miscinfo = 'to';
toexp.N        = 11; toexp.M = [2 3 4 5];
toexp.method   = 'rand'; toexp.orthotype = 'cheby';
toexp.usesamples = roexp;
toexp.addtocrt = expout; toexp.expprefix = 'ortho,';     % only effective in stats+plot mode
if checkrunexp(toexp.miscinfo, exp), expout = lspiexp(toexp); end;

end