function figh = switchedlin_plot(cfg)
% Plot various information regarding the hill car problem
%   FIGH = SWITCHEDLIN_PLOT(CFG)
% Parameters:
%   CFG         - history for a simple trajectory plot;
%               or full configuration of plot, see commented defaults for explanations
%
% Returns:
%   FIGH        - an (array of) handles to the created figure(s)

% default arguments
if nargin < 1, cfg = ''; end;
% % support for default callingmode -- i.e. only history
% if isstruct(cfg) && isfield(cfg, 't') && isfield(cfg, 'x'),
%     hist = cfg;
%     cfg = struct();
%     cfg.trajectory = 1;
%     cfg.datasource = 'caller';
%     cfg.hist = hist;
% end;

% where from to load the data, one of:
%       'caller'    - if the variables should be taken from the calling function
%       filename    - if data should be loaded from a file with name <filename>
CFG.datasource = 'caller';
CFG.datafile   = '';    % data source can be given as a data file here as well
CFG.trajectory = 1;
CFG.hist = [];
% What to plot
% save configuration
CFG.plottarget = 'screen';     % 'latex', 'beamer', 'screen', ''
% only last created figure can be saved using these options
CFG.savedir = ''; % 'D:\Work\tex\papers\alamas07\img\';
                        % path for saving figure
CFG.savefig = 'swlin';    % filename for saving figure
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG

% process config
if ischar(cfg), cfg = str2cfg(cfg, fieldnames(CFG)); end;
cfg = checkparams(cfg, CFG);
cfg.grayscale = grayscalefromconfig(cfg);
% ensure compatibility with datafile field
if ~isempty(cfg.datafile), cfg.datasource = cfg.datafile; end;
% cfg           % feedback on config

% needed vars
% XMFS only required for fuzzy fuzzyh plot
vars = {'model', 'cfg'};

cfg1 = cfg;
% load model, grids, etc. from the data source
switch cfg.datasource,
    case 'caller',              % load needed vars from caller space
        for i=1:length(vars),
            cv.(vars{i}) = evalin('caller', vars{i});
        end;
        structtovars(cv);
    otherwise                   % load from file
        load(cfg.datasource, vars{:});
end;
expcfg = cfg;   % the config from the file
cfg = cfg1;     % the saved config
clear cfg1;     % intermediary variable

% grayscale styles - dummies for now
gs = struct; 
cs = struct;
% set style
if cfg.grayscale, sty = gs; 
else sty = cs; end;

% readable labels
% commonprop = {'Interpreter', 'LaTeX', 'FontSize',13};
labels.time = 'k'; labels.refx = 'x_{ref}'; labels.x = 'x'; labels.u = '\sigma';
styles = {{'Color', [.7 .7 .7], 'LineWidth', 2}, {'-k'}, {'-k'}, {'Color', [.6 .6 .6], 'LineWidth', 2}}; % ref, system, reward

% start plotting
figh = [];

if cfg.trajectory,
    h = cfg.hist;
    if isempty(cfg.addtocrt) || ~cfg.addtocrt, figh(end+1) = figure;
    else figh(end+1) = gcf;
    end;
    subplot(4, 1, [1 2]); if cfg.addtocrt, hold on; end;
    if model.userefmodel, plot(h.t, h.refx(1, :), styles{1}{:}); hold on; end;
    plot(h.t, h.x(1, :), styles{2}{:});
    ylabel(labels.x);
    if model.userefmodel, legend(labels.refx, labels.x); end; % else just one plot so no need
    subplot(4, 1, 3); if cfg.addtocrt, hold on; end;
    stairs(h.t, h.u, styles{3}{:});
    ylim([0.5 2.5]);
    ylabel(labels.u); 
    subplot(7, 1, 7); if cfg.addtocrt, hold on; end;
    plot(h.t, h.r, styles{4}{:});
    ylabel('r');
    xlabel(labels.time); 
	setfigprop(cfg);
end;

% save last figure if indicated
if ~isempty(figh), 
    set(figh(end), 'Name', cfg.datafile, 'NumberTitle', 'off');
    saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
end;

end
% END plot RETURNING array of figure handles =================================================
