%% Numerical Integration
function [xplus] = numerical_integ(m, x, u)
% Inputs:
%   m       - model config
%   x       - current state for current agent
%   u       - action applied for current agent

if m.odemethod(1) == 'v',          % Variable-step ODE
     [odet, odey] = m.odesolver(m.trans, m.odet, x, m.odeopt, m, u);
     xplus = odey(end, :)';

elseif m.odemethod(1) == 'f',      % Fixed-step ODE
     odey = m.odesolver(m.trans, m.odet, x, m, u);
     xplus = odey(end, :)';

elseif m.odemethod(1) == 'e',      % Euler, X(k+1) = X(k) + Ts * f(X(k),u(k)) (possibly multistep)
     dt = m.Ts / m.odesteps; xplus = x;
     for i = 1:m.odesteps,
         xplus = xplus + dt * m.trans(0, xplus, m, u);
     end;
end;