function out = switched_quad(what, varargin)
% Switched linear system where the discrete input is the current mode.
% This function conforms to the specifications established by SAMPLE_PROBLEM.

switch what,
    
    case 'model'    
        % fixed params
        m.id = 'swquad'; 
        m.problem = @switched_quad;
        m.fun = @quad_dyn;
        m.plotfun = [];             % not implemented
        m.q = 1;                    % always one (integer) action: mode
        m.userefmodel = 0;          % no ref model in current implementation
        m.odemethod = 'f';      % 'varode', 'fixedode', 'eul'
        m.odesolver = @ode4;          % solver function
        m.odesteps = 5;               % # of steps per sample time
        m.trans = @dyn;
        
       % configurable params
        PAR.Q = [];                 % cost function matrix
        PAR.goal = [6;1;0;0;0;0];
        PAR.maxx = [];              % state bounds must be estimated by user
        PAR.Ts = .05;               % sample time
        PAR.rewtype = 'opt';        % 'opt'imize (minimize cost/maximize reward) or 'safe'ty bound (maximize cost/minimize reward)
        
        par = parseconfig(varargin{1}, PAR);
        m = copyfields(par, m, fieldnames(par));    % copy params to the model
        
        % process params
        m.M = 5;
        m.U = 1:5;
        m.p = 6;
        m.odet = linspace(0,m.Ts,m.odesteps+1);
        % error checks
        if any(size(m.Q) ~= m.p) || length(m.maxx) ~= m.p, error('Size mismatch');  end;
        % process reward
        if any(any(diag(diag(m.Q)) ~= m.Q)), error('Only diagonal Q supported'); end;
        % if it's diagonal the bound below should work
        maxdiff = max(abs(m.maxx-m.goal), abs(-m.maxx-m.goal));
        m.maxcost = maxdiff' * m.Q * maxdiff;
        m.maxr = 1;

        out = m;
        
    case 'reset',   % handle state reset
        % m = varargin{1};  % argument is not used
        res = varargin{2};
        if ~isnumeric(res), error('Only direct numerical reset supported'); end;
        out = res;

end;


% END doubleint_problem(), RETURNING out ====================================