function batch_tester(testcfg)

%% Test common config

TESTCFG.problem   = 'ip';
TESTCFG.test_type = 'ipoff';
TESTCFG.test_postfix = '';
TESTCFG.algorithms= {};
TESTCFG.nsimarray = [];
TESTCFG.osp_Svals = [1];
TESTCFG.osp_Nvals = [12];
TESTCFG.ovsp_alfa = 1;

testcfg = checkparams(testcfg, TESTCFG);

test_problem    = testcfg.problem;
test_type       = testcfg.test_type;
test_postfix    = testcfg.test_postfix;
algorithms      = testcfg.algorithms;
nsimarray       = testcfg.nsimarray;
OSP_Svals       = testcfg.osp_Svals;
OSP_Nvals       = testcfg.osp_Nvals;
OVSP_alfas      = testcfg.ovsp_alfa;

switch test_problem,
    case 'ip',
        Uval = 2;   Uvalstring = ['_U' num2str(Uval)];    Uvalstring = strrep(Uvalstring,'.','_');
        probl_params = [test_problem test_type Uvalstring test_postfix '_'];
        ccfg = ip_common_config(Uval,'coarse'); %[pi/2; 0]
        Usize = size(ccfg.planparams.U,2);

    case 'hiv',
        probl_params = [test_problem test_type test_postfix '_'];
        ccfg = hiv_common_config();
        Usize = size(ccfg.planparams.U,2);
end;

%% test
for nsim=nsimarray,
    for algo_i = 1:size(algorithms,2),
        algo_name    = algorithms{algo_i}{1};
        algo_version = algorithms{algo_i}{2};
        for OVSP_alfa = OVSP_alfas
            for N=OSP_Nvals,
                for S=OSP_Svals,
                    algo_params = ['_nsim=' num2str(nsim)];
                    cfg = ccfg;           
                    switch algo_name
                        case 'opd'                    
                            cfg.planparams.n = nsim/Usize;
                        case {'osp','usp'}
                            algo_params = [algo_params '_S=' num2str(S)];
                            cfg.planparams.S = S;
                            if ~isempty(strfind(algo_version, 'eqTree'))
                                cfg.planparams.nsim = nsim; 
                            else
                                cfg.planparams.n = nsim/Usize;
                            end
                        case {'osp_oc','osp_oc_reuse'}
                            algo_params = [algo_params '_S=' num2str(S) '_N=' num2str(N)];
                            cfg.planparams.S = S;
                            cfg.planparams.N = N;
                            cfg.planparams.n = nsim/Usize;
                        case {'ovsp','ossp','ossp2','ossp3'}
                            cfg.planparams.n = nsim/Usize;
                            cfg.planparams.S = S;
                            cfg.planparams.alfa = OVSP_alfa;
                            algo_params = [algo_params '_alfa=' num2str(OVSP_alfa*10)];
                            algo_params = strrep(algo_params,'.','_');
                        otherwise
                    end;

                    cfg.planalg = [algo_name algo_version];
                    cfg.datafile = [probl_params algo_name algo_version algo_params];
                    cfg.saveplandata = 1;
                    cfg.visualize = 0;             % this means: only visualize system steps            
                    if strcmp(test_type,'onl'),
                        cfg.online = 1;
                    else
                        cfg.online = 0;
                    end;
                    cfg.run = 1;

                    switch algo_name
                        case 'opd'                    
                            if OVSP_alfa==0.1, genmpc(cfg); end;
                        case {'osp','usp'}
                            if OVSP_alfa==0.1, genmpc(cfg); end;
                        case {'osp_oc','osp_oc_reuse'}
                            genmpc_oc(cfg);
                        case {'ovsp','ossp','ossp2','ossp3'}
                            if S==1, genmpc(cfg); end;
                        otherwise
                    end;
                end;
            end;
        end;
    end;
end;