%% Common config
% close all;
clear all;

Uval = 2;

ccfg = struct;
ccfg.problem = 'ipsetup_problem';
rcfg.rewtype = 'nlqr';
rcfg.Qrew = [1 0];
rcfg.Rrew = 0.1;
ccfg.model_params = {rcfg, ['maxu=' num2str(Uval)], 'Ts=.025'}; 
ccfg.gamma = 0.98;               
ccfg.maxsteps = 120;              % 160 steps = 4 seconds should be enough for a swingup
ccfg.X = 'coarse';              

ccfg.planparams = struct;
ccfg.planparams.U = [-Uval 0 Uval];    % action space -- the same as for M=3 in the approximators above
ccfg.verb = 3;
ccfg.reset = [-pi, 0]';          % always reset pointing down
ccfg.trials = 1;                 % one trial is enough to see performance, as we're not learning
% output config
ccfg.datadir = pwd;
% if using visualization -- set visualize to 0 to disable
ccfg.viscfg.pause = 0.001;      
ccfg.viscfg.figsize = [450 600];


% near opt sol for regret
mfilename = ['_U' num2str(Uval)]; % Ts=0.025 maxstep=160 gamma=0.98
if ~exist(['ip_fzqi_nearopt' mfilename '.mat'], 'file'),
    fzcfg = ccfg;
    fzcfg.problem = 'ipsetup_problem';
    fzcfg.eps = 0.0001;                % convergence threshold
    % state space approximator: an equidistant interpolation grid with N x N elements
    % (the function symequidgrid creates this grid)
    fzcfg.xgrids = symequidgrid(150, [pi; 15*pi]);
    % action space approximator: an equidistant discretization grid with M elements
    fzcfg.ugrids = {ccfg.planparams.U};
    fzcfg.datafile = ['ip_fzqi_nearopt' mfilename];     % save to a data file having this name
    % run the algorithm, remembering the statistics
    fzcfg.run = 1;
    fuzzyqi(fzcfg);
end;

[approx, theta] = fuzzyqi(['approx datafile=ip_fzqi_nearopt' mfilename]);
ccfg.hstar = @(x) approx.h(approx, theta, x);
ccfg.Qstar = @(x, u) approx.q(approx, theta, x, u); 

mfilename = ['ipoff' mfilename]; % Ts=0.025 maxstep=160 gamma=0.98

%% ONLINE OSP_reuse with n
tic;
cfg = ccfg;
cfg.planalg = 'osp_reuse';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 250;         % budget of expansions at each step
cfg.planparams.S = 2;
cfg.datafile = ['iponl_osp_reuse_U' num2str(Uval) '_n=' num2str(cfg.planparams.n) '_S=' num2str(cfg.planparams.S)];
cfg.visualize = 4;
% cfg.saveplandata = 1;

% run the algorithm
cfg.run = 1;
genmpc_reuse(cfg);          

genmpc_reuse(['ploth datafile=' cfg.datafile]);

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));
toc;

% for i=1:60, t1(i)=planinfoh(i).treesize; end;
% for i=1:60, t2(i)=planinfoh(i).treesize; end;
% plot(1:60,t`, 1:60,t2)

%% ONLINE OSP with n
tic;
ccfg.maxsteps = 60; 
cfg = ccfg;
cfg.planalg = 'osp';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 300;         % budget of expansions at each step
cfg.planparams.S = 2;
cfg.datafile = ['iponl_osp_U' num2str(Uval) '_n=' num2str(cfg.planparams.n) '_S=' num2str(cfg.planparams.S)];
cfg.visualize = 4;
cfg.saveplandata = 1;

% run the algorithm
cfg.run = 1;
genmpc(cfg);          

genmpc(['ploth datafile=' cfg.datafile]);

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));
toc;

nuus(1) = max(plandatah{1}.TR);
for i=2:60, nuus(i)=nuus(i-1)+max(plandatah{i}.TR); end;
figure; plot(1:60,nuus);
% for i=1:60, t1(i)=planinfoh(i).treesize; end;
% for i=1:60, t2(i)=planinfoh(i).treesize; end;
% plot(1:60,t`, 1:60,t2)


%% ONLINE OSP with n
tic;
ccfg.maxsteps = 60; 
cfg = ccfg;
cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 300;         % budget of expansions at each step
cfg.planparams.S = 2;
cfg.datafile = ['iponl_opd_U' num2str(Uval) '_n=' num2str(cfg.planparams.n) '_S=' num2str(cfg.planparams.S)];
cfg.visualize = 4;
cfg.saveplandata = 1;

% run the algorithm
cfg.run = 1;
genmpc(cfg);          

genmpc(['ploth datafile=' cfg.datafile]);

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));
toc;

% for i=1:60, t1(i)=planinfoh(i).treesize; end;
% for i=1:60, t2(i)=planinfoh(i).treesize; end;
% plot(1:60,t`, 1:60,t2)


%% ONLINE OSP_OC with n
tic;
cfg = ccfg;
cfg.planalg = 'osp_oc_reuse';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 200;         % budget of expansions at each step
cfg.planparams.S = 2;
cfg.planparams.N = 20; % this is related to ccfg.maxsteps = 60;

cfg.datafile = ['iponl_osp_oc_U' num2str(Uval) '_n=' num2str(cfg.planparams.n) '_S=' num2str(cfg.planparams.S) '_N=' num2str(cfg.planparams.N)];
cfg.visualize = 4;
% cfg.saveplandata = 1;

% run the algorithm
cfg.run = 1;
genmpc_oc(cfg);          

genmpc_oc(['ploth datafile=' cfg.datafile]);

figure; hist=genmpc_oc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));
toc;

% for i=1:60, t1(i)=planinfoh(i).treesize; end;
% for i=1:60, t2(i)=planinfoh(i).treesize; end;
% plot(1:60,t`, 1:60,t2)