function figh = plot_batch(cfg)
% Ad-hoc function to plot a batch of 3 slices of V and h, for:
%   - 4-D state , 2-action problem
%   - obtained with fuzzyqi or LSPI

CFG.datafile = 'fzbike_fail_N19logM3eq';
CFG.alg = 'fuzzy';      % what algorithm was used to produce the data (lspi or fuzzy
CFG.plot = 'hvq';       % plot h (policy), v (value function), or q (Qfunction) slices
CFG.uslice = [0,0];     % for Q-function what action slice to plot
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG

if nargin < 1, cfg = struct; end; 
cfg = parseconfig(cfg, CFG);

figh = [];

if any(cfg.plot == 'h'),
    figh(end+1) = figurex([800 800]);
    postfix = [' plotd gridres=60 datafile=' cfg.datafile ' plottarget=' cfg.plottarget];
    bike_plot([cfg.alg 'h addtocrt=[421,423] slice=[NaN,NaN,0,0] ' postfix]);
    bike_plot([cfg.alg 'h addtocrt=[422,424] slice=[0,0,NaN,NaN] ' postfix]);
    bike_plot([cfg.alg 'h addtocrt=[425,427] slice=[NaN,0,NaN,0] ' postfix]);
    bike_plot([cfg.alg 'h addtocrt=[426,428] slice=[0,NaN,0,NaN] ' postfix]);
end;

if any(cfg.plot == 'v'),
    figh(end+1) = figurex([800 800]);
    postfix = [' addtocrt gridres=40 datafile=' cfg.datafile ' plottarget=' cfg.plottarget];
    subplot(221); bike_plot([cfg.alg 'v slice=[NaN,NaN,0,0] ' postfix]);
    subplot(222); bike_plot([cfg.alg 'v slice=[0,0,NaN,NaN] ' postfix]); 
    subplot(223); bike_plot([cfg.alg 'v slice=[NaN,0,NaN,0] ' postfix]);
    subplot(224); bike_plot([cfg.alg 'v slice=[0,NaN,0,NaN] ' postfix]);
end;

if any(cfg.plot == 'q'),
    figh(end+1) = figurex([800 800]);
    qcfg.([cfg.alg 'q']) = 1;
    qcfg.addtocrt = 1;
    qcfg.datafile = cfg.datafile;
    qcfg.gridres = 40;
    qcfg.plottarget = cfg.plottarget;
    subplot(221); qcfg.slice = [NaN,NaN,0,0, cfg.uslice]; bike_plot(qcfg);
    subplot(222); qcfg.slice = [0,0,NaN,NaN, cfg.uslice]; bike_plot(qcfg); 
    subplot(223); qcfg.slice = [NaN,0,NaN,0, cfg.uslice]; bike_plot(qcfg); 
    subplot(224); qcfg.slice = [0,NaN,0,NaN, cfg.uslice]; bike_plot(qcfg); 
end;

% save last figure if indicated
if ~isempty(figh), saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget); end;
