function [xplus, rplus, terminal] = rarm_mdp(m, x, u)
% Implements the discrete-time dynamics of the robot arm
%  [XPLUS, RPLUS, TERMINAL] = RARM_MDP(M, X, U)
% Forwards the call to rarm_trans in the rarm original directory.
% This function is obsolete. rarm_mdpstandalone should be used instead.
% This function is however required by older model styles.
%
% This function conforms to the specifications established by SAMPLE_MDP.

phys = m.phys;

% limit torque
u = max(-phys.maxtau, min(phys.maxtau, u));

if m.disc.method == 1,          % ODE
    odet = m.disc.odet;           % we only care about solution at final time
	[odet odey] = feval(m.disc.odesolver, @rarm_trans, odet, x, m.disc.odeopt, phys, u);
    xplus = odey(end, :)';      % pick up the state at t + Ts

elseif m.disc.method == 2,      % discrete ODE
    odet = m.disc.odet;
    odey = feval(m.disc.odesolver, @rarm_trans, odet, x, phys, u);
    xplus = odey(end, :)';      % pick up the state at t + Ts
        
elseif m.disc.method == 3,      % Euler, x(k+1) = x(k) + Ts f(x(k), u(k))
    xplus = x + m.disc.Ts * rarm_trans(0, x, phys, u);
end;

% Normalized state
if isfield(phys, 'wrap') && phys.wrap,  % wrap angles in [-pi, pi), bound velocities
    xplus([1 3]) = mod(xplus([1 3]) + pi, 2*pi) - pi;
    xplus([2 4]) = max(-phys.maxomega, min(phys.maxomega, xplus([2 4])));
else        % no wrapping, just bound everything
    xplus = max(-m.maxx, min(m.maxx, xplus));
end;

% Compute reward
if ~isfield(m.goal, 'type') || m.goal.type(1) == 'b',   % box
    %  m.goal reward inside bounds, minimum time reward outside    
    if all(abs(xplus) <= m.goal.tol),
        rplus = m.goal.goalreward;
    else rplus = m.goal.mintimereward;
    end;
elseif m.goal.type(1) == 'l',                           % LQR
    rplus = - x' * m.goal.Q * x  - u' * m.goal.R * u;
end;

terminal = 0;       % task is continuing

% END FUNCTION rarm_mdp() RETURNING xplus, rplus, terminal -----
