function [xplus, rplus, terminal, pplus] = int_mdp(m, x, u)
% Implements the discrete-time dynamics of a simple integrator with
% bounded position, controlled in velocity.
%  [XPLUS, RPLUS, TERMINAl] = INT_MDP(M, X, U)
%
% This function conforms to the specifications established by SAMPLE_MDP.

% limit torque
% no need to limit torque with the current scripts
% u = max(-m.maxu, min(m.maxu, u));

% Compute and bound the next state
xplus = x + m.K * u;
xplus = max(-m.maxx, min(m.maxx, xplus));

% Reward
if m.rewtype(1) == 'l',        % lqr
    rplus = -m.Q * x^2 - m.R * u^2;
elseif m.rewtype(1) == 'n',    % nlqr normalized
    rplus = 1 - (m.Q * x^2 + m.R * u^2) / m.nonnorm_maxr;
elseif m.rewtype(1) == 'm',    % min-time
    rplus = m.zeroreward * (abs(x) <= m.zeroband);
else error('Reward type %s unknown', m.rewtype);
end;

terminal = 0;       % task is continuing
if nargout == 4, pplus = 1; end;    % probability

% END FUNCTION int_mpd() RETURNING xplus, rplus ==================================================
