
clear setups evol

% setups.ode45 = 'method=ode odesolver=ode45';
% setups.ode15s = 'method=ode odesolver=ode15s';
setups.ode4Nsteps10 = 'method=fixed-ode odesolver=ode4 odesteps=10';
% setups.ode4Nsteps5 = 'method=fixed-ode odesolver=ode4 odesteps=5';
% setups.ode4Nsteps4 = 'method=fixed-ode odesolver=ode4 odesteps=4';
% setups.ode4Nsteps3 = 'method=fixed-ode odesolver=ode4 odesteps=3';
% setups.ode4Nsteps2 = 'method=fixed-ode odesolver=ode4 odesteps=2';
% setups.ode4Nsteps1 = 'method=fixed-ode odesolver=ode4 odesteps=1';
% setups.eulerNsteps100 = 'method=euler odesteps=100';
% setups.ode5Nsteps10 = 'method=fixed-ode odesolver=ode5 odesteps=10';
% setups.ode3Nsteps1 = 'method=fixed-ode odesolver=ode3 odesteps=1';
% setups.ode2Nsteps1 = 'method=fixed-ode odesolver=ode2 odesteps=1';
% setups.ode1Nsteps1 = 'method=fixed-ode odesolver=ode1 odesteps=1';
setups.traj = setups.ode4Nsteps10;

% preinit inputs and initial state
maxsteps = 50;
x0 = [0 0]';
useq = 4 + zeros(1, maxsteps);

useq(rand(1, maxsteps) <= .5) = -4;

% useq(2:2:length(useq)) = -4;

% useq(2:4:length(useq)) = -4;
% useq(3:4:length(useq)) = -4;

% evolution storage
evol = struct;

% create & setup figure
figh = figurex([1000 800]);
styles = {{'k--','LineWidth',2,'Color',[.6,.6,.6]}, {'k-','LineWidth',1}, ...
    {'k:','LineWidth',1,'Color',[0,1,0]}, {'k--','LineWidth',1,'Color',[0,0,1]}, ...
    {'k:','LineWidth',1,'Color',[1,0,0]}, {'k--','LineWidth',1,'Color',[.5,.5,0]}};

names = fieldnames(setups); Nsmax = -inf; leg = {};
for i = 1:length(names),
    % create model
    model = hillcar_problem('model', setups.(names{i}));
    
    % init history
    t = (0 : maxsteps) * model.Ts;
    Ns = length(t) - 1;
    x = zeros(model.p, length(t)); x(:, 1) = x0;
    u = zeros(model.q, length(t)); u(:, 1:end-1) = useq; u(:, end) = NaN;
    r = zeros(1, length(t)); r(1) = NaN;

    if strcmp(names{i}, 'traj'),
        tmark = cputime;
        [X R Ns] = hillcar_traj_ode4(model, x0, u, maxsteps);
        hist.x = X;
        hist.r = R;
        trun = cputime - tmark;
    else
        % steps loop
        tmark = cputime;
        for k = 1:Ns,
            % apply to system
            [x(:, k+1) r(k+1) terminal] = feval(model.fun, model, x(:, k), u(:, k));
            if terminal, Ns = k; break; end;      % entered terminal state
        end;
        trun = cputime - tmark;

        % form and save history
        hist.t = t(1:Ns+1); hist.x = x(:, 1:Ns+1); hist.u = u(:, 1:Ns+1); hist.r = r(1:Ns+1);    
        evol.(names{i}) = hist;
        Nsmax = max(Ns, Nsmax);
    end;

    si = mod(i, length(styles)) + 1;
    subplot(312); plot(hist.t, hist.x(1, :), styles{si}{:}); hold on;
    subplot(313); plot(hist.t, hist.x(2, :), styles{si}{:}); hold on;
    
    leg{end+1} = [names{i} ',t_{run}=' num2str(trun*1000)];
%     pause;
end;


% finalize plots
subplot(311); stairs((0:Nsmax) * model.Ts, [useq(1:Nsmax) NaN]); hold on;   % last command wasn't applied
ylabel('u(kT)');
subplot(312); ylabel('p(kT)'); legend(leg, 'Location', 'Best');
subplot(313); ylabel('s(kT)'); xlabel('kT'); legend(leg, 'Location', 'Best');


clear x u r t Ns