%% Initialize
clear all; 
% close all;

% domain
maxx = [2 2]'; % dimension of (symmetrical) input space
D = length(maxx);   % dimensionality

% database samples
% Ns = 5000;           % number of samples to build the tree from
% Xs = rand(D, Ns) .* repmat(2*maxx, 1, Ns) - repmat(maxx, 1, Ns);
% Us = zeros(1, Ns);  % dummy action points

Ns = [100 100];           % number of samples to build the tree from
XXs = symequidgrid(Ns, maxx);
Xs = flat(XXs);
Us = zeros(1, prod(Ns));  % dummy action points

% query samples
Nq = [37 37];            % number of query samples
XXq = symequidgrid(Nq, maxx*.9);
Xq = flat(XXq);
Uq = zeros(1, prod(Nq));

% main approx config
cfg.U = {0};            % dummy action space
cfg.Xs = Xs;
cfg.Us = Us;
cfg.knnmethod = 'jit'; 
cfg.knn = 10;
cfg.S = 1 ./ maxx;            
cfg.linsolvemethod = 'pinv';  
cfg.extrap = 'sat';           

% dummy model
model.maxx = maxx;
model.maxu = 0;
model.p = D;
model.q = 1;

% some functions to test
A = [1 .5]; b = 2; 
linfun = @(x) [1 .5] * x + 2;
banana = @(x) 100.*(x(2, :)-x(1, :).^2).^2+(1-x(1, :)).^2;


%% Customize test
testfun = banana;       % function
cfg.wfun = 'cube';      % weighting

%% Run test and output results
cfg.Qs = testfun(Xs)';
approx = llrdisc(model, cfg);

Yq = approx.q(approx, [], Xq, Uq);
Yt = testfun(Xq);                   % true values

lims = minmax(Yt);

% Output results
figurex([800 400]);
subplot(121); mesh(XXq{1}, XXq{2}, reshape(Yt, Nq)'); 
% view([-9 30]); 
zlim(lims);
zlabel('Exact');
subplot(122); mesh(XXq{1}, XXq{2}, reshape(Yq, Nq)'); 
% view([-9 30]); 
zlim(lims);
zlabel(sprintf('Approx, err=%.4f', mean((Yt-Yq).^2)));
set(gcf, 'Name', sprintf('wfun=%s, extrap=%s', cfg.wfun, cfg.extrap));

