%% General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planalg', 'X', 'planparamsn', 'planparamsCp'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithm
ACFG = struct;
ACFG.problem = 'dc_problem';
ACFG.model_params = {'nlqr', 'type=lin stoch_prob=[.6,.4] stoch_gain=[1,.7] Ts=0.05'};
ACFG.online = 0;
ACFG.gamma = 0.95;
ACFG.planalg = 'uct';
ACFG.planparams.L = 2; % although not directly used
ACFG.planparams.U = [-10 0 10];
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 20;
ACFG.reset = [-pi, 0]';
% visualization
ACFG.verb = 3;
ACFG.datadir = 'D:\Work\m\approxrl\testing\test_uct';
ACFG.X = 'coarse';  % results also available for "coarser"

%% Near-optimal solution: fuzzyQI -- run if not present

cfg = ACFG;
dummymodel = dc_problem('model', cfg.model_params{:});
cfg.xgrids = symequidgrid(201, dummymodel.maxx);
cfg.ugrids = {cfg.planparams.U};
cfg.maxiter = 1000;
cfg.eps = 1e-5;
cfg.datafile = 'fzqi_dc_nearopt_p04g07_Ts005';

% add near-optimal solution to default config if need to compute regret
if ~exist([cfg.datafile '.mat'], 'file'),
    cfg.run = 1;
    fuzzyqi(cfg);
end;
[approx, theta] = fuzzyqi(['approx datafile=' cfg.datafile]);
ACFG.hstar = @(x) approx.h(approx, theta, x);
ACFG.Qstar = @(x, u) approx.q(approx, theta, x, u); 


%% UCT - varying n

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.id = 'uct';   % note we don't use this as an infix -- it already appears as algorithm name
bcfg.legendprefix = 'UCT';
bcfg.repeat = 1;
bcfg.plotconf = 0;  % the interval is so tight it's useless
% transform input to align it with the OPSS "n"
% bcfg.intransform = @(val) round(val ./ ACFG.planparams.L ./ length(ACFG.planparams.U));

acfg = ACFG;
acfg.planalg = 'uct';
acfg.planparams.n = [100 300 500 700 1000 2000] * ACFG.planparams.L * length(ACFG.planparams.U);
acfg.planparams.Cp = 1;


%% run
processbatch(bcfg, acfg); 

%% eval
bcfg.in = 'planparamsn';
bcfg.out = 'regret';
evalbatch(bcfg, acfg);


%% UCT Online

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'uctonl';
% bcfg.legendprefix = 'Uniform';

acfg = ACFG;
acfg.online = 1;
acfg.planalg = 'uct';
acfg.planparams.n = 5000;
acfg.planparams.Cp = 1;

processbatch(bcfg, acfg);
