function [figh, varargout] = fuzzyqi_resstudy(cfg)
% Study of Q-iteration as the number of (equidistant) fuzzy centers on each axis increases
% Assumes hyper-rectangular Euclidean state space with bounds in model.maxx
% Filenames created assuming a 2-dimensional state space 
%
% THIS FUNCTION IS OBSOLETE AND HAS BEEN REPLACED WITH QIRESSTUDY

if nargin < 1,
    cfg = struct;
end;

CFG.res = 3:50;         % span of cfg.resolutions to try out
CFG.problem = 'ddi_problem';
CFG.gamma = 0.95;
CFG.eps = 1e-7;
CFG.maxiter = 1000;
CFG.datafile = '';
CFG.run = 1;
CFG.initstepdisp = .1;
CFG.itersave = 10;
CFG.X0 = 'fine';
CFG.silent = 1;
CFG.mc_eps = 0.01;
CFG.grayscale = 1;
CFG.algorithms = {'fz'};
% Default reward type for DDI problem
% kept for compatibility with older calls to this function, that assumed it's hardcoded for DDI
% problem. Should be explicitly overwritten with appropriate arguments (or empty cell array) if other
% problems are used
CFG.model_params = {'quad'};

% what to plot
CFG.perf = 1;
CFG.runtime = 0;
CFG.iter = 0;
% plot config
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG
CFG.logy = 0;
CFG.Jstar = [];
CFG.eachaxis = 0;   % do not report total number of basis functions, but only on 1 axis
CFG.uniformxticks = 0;
% cfg.model_params = {};
% cfg.algorithms = {'fz', 'rbf'};
% cfg.algorithms = {'rbf'};

% save configuration
CFG.plottarget = 'screen';     % 'latex', 'beamer', 'screen', ''
% only last created figure can be saved using these options
CFG.savedir = ''; % 'D:\Work\tex\papers\alamas07\img\';
                        % path for saving figure
CFG.savefig = 'fzqires';    % filename for saving figure

% process config
if ischar(cfg), cfg = str2cfg(cfg, fieldnames(CFG)); end;
cfg = checkparams(cfg, CFG);
cfg.grayscale = grayscalefromconfig(cfg);
if ~iscell(cfg.model_params),
    cfg.model_params = {cfg.model_params};
end;
nexp = length(cfg.res); % number of experiments

% echo
cfg.N = cfg.res + 1;      % number of bfuns on each axis is always res+1
cfg

% precision 0.01 in MC return evaluation NOTE max reward hardcoded
model = feval(cfg.problem, 'model', cfg.model_params{:});
cfg.mc_maxsteps = ceil(log(cfg.mc_eps * (1-cfg.gamma) / model.maxr) / log(cfg.gamma));

cecfg = feval(cfg.problem, 'ce', cfg.X0);
cfg.U = cecfg.U;
fz.J = nan(nexp, size(cecfg.X0, 2));
fz.eJ = nan(nexp, 1); fz.runtime = nan(nexp, 1); fz.iter = nan(nexp, 1);
rbf = fz; rbf.br = nan(nexp, 1); rbf.iter = nan(nexp, 1);
% problem prefix in data files
switch cfg.problem
    case 'ddi_problem',
        ppref = 'dd_';
    case 'hillcar_problem',
        ppref = 'hillcar_';
end;
        
for i = 1:nexp,
    r = cfg.res(i);
    % form xgrids
    xgrids = cell(model.p, 1);
    for p = 1:model.p,
        xgrids{p} = -model.maxx(p):(2*model.maxx(p)/r):model.maxx(p);
    end;

    df = ppref;
    if ~isempty(cfg.model_params) && ischar(cfg.model_params{1}),
        df = [df cfg.model_params{1} '_'];
    end;
    
    % first, fuzzy Q-iteration
    if any(strcmp(cfg.algorithms, 'fz')),
        cfg.xgrids = xgrids;
        cfg.datafile = [df num2str(r) 'x' num2str(r) '_fuzzy'];
        if ~exist([cfg.datafile '.mat'], 'file'),
            disp(['Running fuzzy Q-iter res=' num2str(r)]);
            fuzzyqi(cfg);
        end;
        load(cfg.datafile, 'theta', 'model', 'XMFS', 'qistats'); thetastar = theta; clear theta;
        if cfg.perf,    % only perform monte-carlo if perf plot is requested
            cfg.interph = 0;
            [fz.eJ(i) fz.J(i, :)] = mc_fuzzyq(XMFS, thetastar, model, cecfg.X0, cfg);
        end;
        fz.runtime(i) = qistats.t + qistats.tinit;
        % #iter computed from the delta record
        fz.iter(i) = length(qistats.delta);
    end;

    % RBF Q-iter not consistent or bug in the code, this is not climbing to the optimal
    % performance
    if any(strcmp(cfg.algorithms, 'rbf')),
        % then, RBF Q-iteration
        cfg.c = xgrids;
        cfg.rad = [2/r 1/r];   % radii  = grid spacing
        cfg.ugrids = {[-1 1]};
        cfg.datafile = [df num2str(r) 'x' num2str(r) '_rbf'];
        if ~exist([cfg.datafile '.mat'], 'file'),
            thetastar = rbfqi(cfg);
        end;
        load(cfg.datafile, 'thetastar', 'model', 'C', 'RAD', 'qistats');
        if cfg.perf,    % only perform monte-carlo if perf plot is requested
            [rbf.eJ(i) rbf.J(i, :)] = mc_rbfq(C, RAD, thetastar, model, cecfg.X0, cfg);
        end;
        brcfg.bellmancfg.res = 1; brcfg.datafile = cfg.datafile; brcfg.X0 = cecfg.X0; brcfg.silent = 1;
        rbf.br(i) = rbfqi(brcfg);
        rbf.runtime(i) = qistats.t + qistats.tinit;
        rbf.iter(i) = length(qistats.delta);
    end;
    
end;

% ----------------------
% Plot code
% grayscale styles
gs.fz = {'--ko', 'MarkerSize', 3, 'LineWidth', 1.3};
gs.rbf = {':ks', 'MarkerSize', 3, 'LineWidth', 1.3};
gs.optstyle = {'-', 'Color', 'k', 'LineWidth', 2};
gs.cm = gray(64);
% color styles
cs = gs;    % by default equal to the grayscale
cs.cm = jet(64);
% set style
if cfg.grayscale, sty = gs; 
else sty = cs; end;

% readable labels
commonprop = {'Interpreter', 'LaTeX', 'FontSize',13};
if cfg.eachaxis,
    rl.x = {'Number of fuzzy sets on each axis'};
else
    rl.x = {'Number of fuzzy sets \it{N}'};
end;
rl.perf = {'Score'};
rl.runtime = {'Execution time [s]'};
rl.iter = {'Iterations '};
rl.fzperf = 'fuzzy Q-iteration score';
rl.fzruntime = 'fuzzy Q-iteration execution time';
rl.fziter = '# of fuzzy Q-iterations';
rl.rbfperf = 'RBF Q-iteration score';
rl.rbfruntime = 'RBF Q-iteration execution time';
rl.rbfiter = '# of RBF Q-iterations';
rl.opt = 'optimal score';
labels = rl;    % no psfrag labels

figh = [];

if cfg.perf,
    figh(end+1) = figure; hold on; box on; leg = {};
    for i = 1:length(cfg.algorithms),
        data = eval(cfg.algorithms{i});     % the variable name, fz, rbf etc.
        plot(cfg.N, data.eJ, sty.(cfg.algorithms{i}){:}); 
        leg{end+1} = labels.([cfg.algorithms{i} 'perf']);
    end;
    if ~isempty(cfg.Jstar),
        plot([cfg.N(1) cfg.N(end)], [cfg.Jstar cfg.Jstar], sty.optstyle{:});
        leg{end+1} = labels.opt;
    end;
    setfigprop(cfg);
    annotatefig('perf', cfg, leg, labels, commonprop);
    varargout = {data.eJ};
end;

if cfg.runtime,
    figh(end+1) = figure; box on; leg = {};
    if cfg.logy, plotfun = @semilogy;
    else plotfun = @plot;
    end;
    for i = 1:length(cfg.algorithms),
        data = eval(cfg.algorithms{i});     % the variable name, fz, rbf etc.
        feval(plotfun, cfg.N, data.runtime, sty.(cfg.algorithms{i}){:}); 
        hold on;  % moved here so that the Y axis actually becomes logarithmic
        leg{end+1} = labels.([cfg.algorithms{i} 'runtime']);
    end;
    hold on;
    setfigprop(cfg);
    annotatefig('runtime', cfg, leg, labels, commonprop);
end;

if cfg.iter,
    figh(end+1) = figure; hold on; box on; leg = {};
    if cfg.logy, plotfun = @semilogy;
    else plotfun = @plot;
    end;
    for i = 1:length(cfg.algorithms),
        data = eval(cfg.algorithms{i});     % the variable name, fz, rbf etc.
        feval(plotfun, cfg.N, data.iter, sty.(cfg.algorithms{i}){:}); 
        leg{end+1} = labels.([cfg.algorithms{i} 'iter']);
    end;
    hold on;
    cfg.ylim
    setfigprop(cfg);
    annotatefig('iter', cfg, leg, labels, commonprop);
end;

% save last figure if indicated
if ~isempty(figh), 
    set(figh(end), 'Name', cfg.datafile, 'NumberTitle', 'off');
    saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
end;


end     % fuzzyqi_resolutionstudy =============================================


% --------------------
% Local functions


function setlimits(cfg)
    if ~isempty(cfg.xlim),
        set(gca, 'Xlim', cfg.xlim);
    end;
    if ~isempty(cfg.ylim),
        set(gca, 'Ylim', cfg.ylim);
    end;
end

function annotatefig(what, cfg, leg, labels, commonprop)

xlabel(labels.x{:});
ylabel(labels.(what){:});
N =  cfg.N;
if ~cfg.uniformxticks,
    cfg.tick = [N(1):3:12 15:5:N(end)];
    set(gca, 'XGrid', 'on');
else
    cfg.tick = get(gca, 'XTick');
end;
set(gca, 'XTick', cfg.tick);
tl = {}; 
if ~cfg.eachaxis,
    for i = cfg.tick; 
        tl{end+1} = [num2str(i) '^2']; 
    end;
    set(gca, 'XTickLabel', tl); set(twxticklabel, 'Interpreter', 'TeX');
end;
lh = legend(leg);
set(lh, 'Location', 'Best');

end     % annotatefig
