% Compute near-optimal solution for the hillcar using fuzzyqi;
% also find Monte-Carlo score for some solutions

%% New version taking into account the terminal state
clear cfg

cfg.problem = 'hillcar_problem';
cfg.xgrids = {-1:.005:1, -3:.02:3};
cfg.gamma = 0.95;
cfg.eps = 1e-7;
cfg.maxiter = 1000;
cfg.datafile = 'hc_nearopt_fuzzy_termzero';
cfg.run = 1;
cfg.initstepdisp = .05;
cfg.itersave = 10;
cfg.term = 'zero';
cfg.X0 = 'posspeed';

if ~exist([cfg.datafile '.mat'], 'file'),
    thetastar = fuzzyqi(cfg);
end;

mccfg = struct; 
mccfg.datafile = 'hc_nearopt_fuzzy_termzero';
mccfg.mc_eps = 0.01;
cfg2 = hillcar_problem('ce', cfg.X0);
mccfg.gamma = cfg.gamma;
mccfg.X0 = cfg2.X0; clear cfg2;
[eJ, J] = mc_fuzzyqi(mccfg);


%% Old version, ignoring terminal state
clear cfg

cfg.problem = 'hillcar_problem';
cfg.xgrids = {-1:.005:1, -3:.02:3};
cfg.gamma = 0.95;
cfg.eps = 1e-7;
cfg.maxiter = 1000;
cfg.datafile = 'hc_nearopt_fuzzy_ode4N10';
cfg.run = 1;
cfg.initstepdisp = .05;
cfg.itersave = 10;
cfg.X0 = 'posspeed';

if ~exist([cfg.datafile '.mat'], 'file'),
    thetastar = fuzzyqi(cfg);
end;

mccfg = struct; 
mccfg.datafile = 'hc_nearopt_fuzzy_ode4N10';
mccfg.mc_eps = 0.01;
cfg2 = hillcar_problem('ce', cfg.X0);
mccfg.gamma = cfg.gamma;
mccfg.X0 = cfg2.X0; clear cfg2;
[eJ, J] = mc_fuzzyqi(mccfg);


%% Stochastic hillcar, larger sampling time, progress-based reward function (with terminal states zeroed)
clear cfg

cfg.problem = 'hillcar_problem';
cfg.model_params = {'rewtype=prog Ts=.2 stoch_prob=[.5,.5] stoch_gain=[1.2,.8]'};
cfg.xgrids = {-1:.01:1, -3:.05:3};   % note the lower resolution than above
cfg.gamma = 0.95;
cfg.eps = 1e-7;
cfg.maxiter = 1000;
cfg.datafile = 'hc_nearopt_fuzzy_stoch_prog_Ts02';
cfg.run = 1;
cfg.initstepdisp = .05;
cfg.itersave = 100;
cfg.term = 'zero';
cfg.X0 = 'posspeed';

thetastar = fuzzyqi(cfg);




